/* $Id: ikev1.c,v 1.6 2005/10/06 14:54:23 inoue Exp $ */

/*
 * Copyright (C) 2004 WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

static int nostate1 __P((struct ph1handle *, vchar_t *));
static int nostate2 __P((struct ph2handle *, vchar_t *));

extern caddr_t val2str(const char *, size_t);

static int ph1_main __P((struct ph1handle *, vchar_t *));
static int quick_main __P((struct ph2handle *, vchar_t *));
static int isakmp_ph1begin_r __P((vchar_t *,
				  struct sockaddr *, struct sockaddr *,
				  u_int8_t));
static int isakmp_ph2begin_i __P((struct ph1handle *, struct ph2handle *));
static int isakmp_ph2begin_r __P((struct ph1handle *, vchar_t *));

typedef int (*PH1EXCHG) __P((struct ph1handle *, vchar_t *));

PH1EXCHG ph1exchange[][2][PHASE1ST_MAX] = {
	/* error */
	{{}, {},},
	/* Identity Protection exchange */
	{
	 {nostate1, ident_i1send, nostate1, ident_i2recv, ident_i2send,
	  ident_i3recv, ident_i3send, ident_i4recv, ident_i4send, nostate1,},
	 {nostate1, ident_r1recv, ident_r1send, ident_r2recv, ident_r2send,
	  ident_r3recv, ident_r3send, nostate1, nostate1, nostate1,},
	 },
	/* Aggressive exchange */
	{
	 {nostate1, agg_i1send, nostate1, agg_i2recv, agg_i2send, nostate1,
	  nostate1, nostate1, nostate1, nostate1,},
	 {nostate1, agg_r1recv, agg_r1send, agg_r2recv, agg_r2send, nostate1,
	  nostate1, nostate1, nostate1, nostate1,},
	 },
	/* Base exchange */
	{
	 {nostate1, base_i1send, nostate1, base_i2recv, base_i2send,
	  base_i3recv, base_i3send, nostate1, nostate1, nostate1,},
	 {nostate1, base_r1recv, base_r1send, base_r2recv, base_r2send,
	  nostate1, nostate1, nostate1, nostate1, nostate1,},
	 },
};

typedef int (*PH2EXCHG) __P((struct ph2handle *, vchar_t *));

PH2EXCHG ph2exchange[][2][PHASE2ST_MAX] = {
	/* error */
	{{}, {},},
	/* Quick mode for IKE */
	{
	 {nostate2, nostate2, quick_i1prep, nostate2, quick_i1send,
	  quick_i2recv, quick_i2send, quick_i3recv, nostate2, nostate2,},
	 {nostate2, quick_r1recv, quick_r1prep, nostate2, quick_r2send,
	  quick_r3recv, quick_r3prep, quick_r3send, nostate2, nostate2,},
	 },
};

static int etypesw1 __P((int));
static int etypesw2 __P((int));

struct dh_def ikev1_dhdef = {
	{algtype_dhg_modp768, OAKLEY_ATTR_GRP_DESC_MODP768, &dh_mopd768},
	{algtype_dhg_modp1024, OAKLEY_ATTR_GRP_DESC_MODP1024, &dh_modp1024},
	/* { algtype_dhg_ec2n155, OAKLEY_ATTR_GRP_DESC_EC2N155, .... }, */
	/* { algtype_dhg_ec2n185, OAKLEY_ATTR_GRP_DESC_EC2N185, .... }, */
	{algtype_dhg_modp1536, OAKLEY_ATTR_GRP_DESC_MODP1536, &dh_modp1536},
	/* ec2n_163_a */
	/* ec2n_163_b */
	/* ec2n_283_a */
	/* ec2n_283_b */
	/* ec2n_409_a */
	/* ec2n_409_b */
	/* ec2n_571_a */
	/* ec2n_571_b */
	{algtype_dhg_modp2048, OAKLEY_ATTR_GRP_DESC_MODP2048, &dh_modp2048},
	{algtype_dhg_modp3072, OAKLEY_ATTR_GRP_DESC_MODP3072, &dh_modp3072},
	{algtype_dhg_modp4096, OAKLEY_ATTR_GRP_DESC_MODP4096, &dh_modp4096},
	{algtype_dhg_modp6144, OAKLEY_ATTR_GRP_DESC_MODP6144, &dh_modp6144},
	{algtype_dhg_modp8192, OAKLEY_ATTR_GRP_DESC_MODP8192, &dh_modp8192},
	{0}
};

/*
 * main processing to handle isakmp payload
 */
static int
isakmp_main(msg, remote, local)
	vchar_t *msg;
	struct sockaddr *remote, *local;
{
	struct isakmp *isakmp = (struct isakmp *)msg->v;
	isakmp_index_t *index = (isakmp_index_t *)isakmp;
	u_int32_t msgid = isakmp->msgid;
	struct ph1handle *iph1;

	++isakmpstat.v1input;

#ifdef HAVE_PRINT_ISAKMP_C
	isakmp_printpacket(msg, remote, local, 0);
#endif

	/* XXX: check sender whether to be allowed or not to accept */

	/* XXX: I don't know how to check isakmp half connection attack. */

	/* simply reply if the packet was processed. */
	if (check_recvdpkt((struct sockaddr *)&remote,
			   (struct sockaddr *)&local, msg)) {
		plog(PLOG_INFO, LOCATION, remote,
		     "the packet is retransmitted by %s.\n",
		     saddr2str((struct sockaddr *)&remote));
		++isakmpstat.duplicate;
		return 0;
	}

	/* (RFC2408) 
	 * Implementations SHOULD never accept packets with a minor
	 * version number larger than its own, given the major version
	 * numbers are identical.
	 */
	if (ISAKMP_GETMINORV(isakmp->v) > ISAKMP_MINOR_VERSION) {
		plog(PLOG_PROTOERR, LOCATION, remote,
		     "unsupported isakmp version %d.%03d.\n",
		     ISAKMP_GETMAJORV(isakmp->v), ISAKMP_GETMINORV(isakmp->v));
		/* XXX should send notification */
		++isakmpstat.unsupported_version;
		return -1;
	}

	/* the initiator's cookie must not be zero */
	if (memcmp(&isakmp->i_ck, r_ck0, sizeof(isakmp_cookie_t)) == 0) {
		plog(LLV_ERROR, LOCATION, remote,
		     "malformed cookie received.\n");
		++isakmpstat.invalid_ike_spi;
		return -1;
	}

	/* check the Flags field. */
	/* XXX How is the exclusive check, E and A ? */
	if (isakmp->flags & ~(ISAKMP_FLAG_E | ISAKMP_FLAG_C | ISAKMP_FLAG_A)) {
		plog(LLV_ERROR, LOCATION, remote,
		     "invalid flag 0x%02x.\n", isakmp->flags);
		++isakmpstat.invalid_flag;
		return -1;
	}

	/* ignore commit bit. */
	if (ISSET(isakmp->flags, ISAKMP_FLAG_C)) {
		if (isakmp->msgid == 0) {
			isakmp_info_send_nx(isakmp, remote, local,
					    ISAKMP_NTYPE_INVALID_FLAGS, NULL);
			plog(LLV_ERROR, LOCATION, remote,
			     "Commit bit on phase1 forbidden.\n");
			++isakmpstat.invalid_flag;
			return -1;
		}
	}

	iph1 = getph1byindex(index);
	if (iph1 != NULL) {
		/* validity check */
		if (memcmp(&isakmp->r_ck, r_ck0, sizeof(isakmp_cookie_t)) == 0
		    && iph1->side == INITIATOR) {
			plog(LLV_DEBUG, LOCATION, remote,
			     "malformed cookie received or "
			     "the initiator's cookies collide.\n");
			++isakmpstat.invalid_ike_spi;
			return -1;
		}

		/* must be same addresses in one stream of a phase at least. */
		if (cmpsaddrstrict(iph1->remote, remote) != 0) {
			char *saddr_db, *saddr_act;

			saddr_db = strdup(saddr2str(iph1->remote));
			saddr_act = strdup(saddr2str(remote));

			plog(LLV_WARNING, LOCATION, remote,
			     "remote address mismatched. db=%s, act=%s\n",
			     saddr_db, saddr_act);

			racoon_free(saddr_db);
			racoon_free(saddr_act);
		}
		/*
		 * don't check of exchange type here because other type will be
		 * with same index, for example, informational exchange.
		 */

		/* XXX more acceptable check */
	}

	switch (isakmp->etype) {
	case ISAKMP_ETYPE_IDENT:	/* == oakley main mode */
	case ISAKMP_ETYPE_AGG:
	case ISAKMP_ETYPE_BASE:
		/* phase 1 validity check */
		if (isakmp->msgid != 0) {
			plog(LLV_ERROR, LOCATION, remote,
			     "message id should be zero in phase1.\n");
			++isakmpstat.invalid_message_id;
			return -1;
		}

		/* search for isakmp status record of phase 1 */
		if (iph1 == NULL) {
			/*
			 * the packet must be the 1st message from a initiator
			 * or the 2nd message from the responder.
			 */

			/* search for phase1 handle by index without r_ck */
			iph1 = getph1byindex0(index);
			if (iph1 == NULL) {
				/*it must be the 1st message from a initiator. */
				if (memcmp(&isakmp->r_ck, r_ck0,
					   sizeof(isakmp_cookie_t)) != 0) {

					plog(LLV_DEBUG, LOCATION, remote,
					     "malformed cookie received "
					     "or the spi expired.\n");
					++isakmpstat.unknown_cookie;
					return -1;
				}

				/* it must be responder's 1st exchange. */
				if (isakmp_ph1begin_r(msg, remote, local,
						      isakmp->etype) < 0)
					return -1;
				break;

			 /*NOTREACHED*/}

			/* it must be the 2nd message from the responder. */
			if (iph1->side != INITIATOR) {
				plog(LLV_DEBUG, LOCATION, remote,
				     "malformed cookie received. "
				     "it has to be as the initiator.  %s\n",
				     isakmp_pindex(&iph1->index, 0));
				++isakmpstat.invalid_message_id;
				return -1;
			}
		}

		/*
		 * Don't delete phase 1 handler when the exchange type
		 * in handler is not equal to packet's one because of no
		 * authencication completed.
		 */
		if (iph1->etype != isakmp->etype) {
			plog(LLV_ERROR, LOCATION, iph1->remote,
			     "exchange type is mismatched: "
			     "db=%s packet=%s, ignore it.\n",
			     s_isakmp_etype(iph1->etype),
			     s_isakmp_etype(isakmp->etype));
			++isakmpstat.unexpected_packet;
			return -1;
		}

		/* call main process of phase 1 */
		if (ph1_main(iph1, msg) < 0) {
			plog(LLV_ERROR, LOCATION, iph1->remote,
			     "phase1 negotiation failed.\n");
			remph1(iph1);
			delph1(iph1);
			return -1;
		}
		break;

#if 0
	case ISAKMP_ETYPE_AUTH:
		plog(LLV_INFO, LOCATION, remote,
		     "unsupported exchange %d received.\n", isakmp->etype);
		++isakmpstat.unsupported_exchange_type;
		break;
#endif

	case ISAKMP_ETYPE_INFO:
	case ISAKMP_ETYPE_ACKINFO:
		/*
		 * iph1 must be present for Information message.
		 * if iph1 is null then trying to get the phase1 status
		 * as the packet from responder againt initiator's 1st
		 * exchange in phase 1.
		 * NOTE: We think such informational exchange should be ignored.
		 */
		if (iph1 == NULL) {
			iph1 = getph1byindex0(index);
			if (iph1 == NULL) {
				plog(LLV_ERROR, LOCATION, remote,
				     "unknown Informational "
				     "exchange received.\n");
				++isakmpstat.infoexch_unknown_peer;
				return -1;
			}
			if (cmpsaddrstrict(iph1->remote, remote) != 0) {
				plog(LLV_WARNING, LOCATION, remote,
				     "remote address mismatched. "
				     "db=%s\n", saddr2str(iph1->remote));
				++isakmpstat.infoexch_unknown_remote_addr;
			}
		}

		if (isakmp_info_recv(iph1, msg) < 0)
			return -1;
		break;

	case ISAKMP_ETYPE_QUICK:
		{
			struct ph2handle *iph2;

			if (iph1 == NULL) {
				isakmp_info_send_nx(isakmp, remote, local,
						    ISAKMP_NTYPE_INVALID_COOKIE,
						    NULL);
				plog(LLV_ERROR, LOCATION, remote,
				     "can't start the quick mode, "
				     "there is no ISAKMP-SA, %s\n",
				     isakmp_pindex((isakmp_index_t *)&isakmp->
						   i_ck, isakmp->msgid));
				++isakmpstat.invalid_ike_spi;
				return -1;
			}

			/* check status of phase 1 whether negotiated or not. */
			if (iph1->status != PHASE1ST_ESTABLISHED) {
				plog(LLV_ERROR, LOCATION, remote,
				     "can't start the quick mode, "
				     "there is no valid ISAKMP-SA, %s\n",
				     isakmp_pindex(&iph1->index, iph1->msgid));
				++isakmpstat.premature;
				return -1;
			}

			/* search isakmp phase 2 stauts record. */
			iph2 = getph2bymsgid(iph1, msgid);
			if (iph2 == NULL) {
				/* it must be new negotiation as responder */
				if (isakmp_ph2begin_r(iph1, msg) < 0)
					return -1;
				return 0;
			 /*NOTREACHED*/}

			/* commit bit. */
			/* XXX
			 * we keep to set commit bit during negotiation.
			 * When SA is configured, bit will be reset.
			 * XXX
			 * don't initiate commit bit.  should be fixed in the future.
			 */
			if (ISSET(isakmp->flags, ISAKMP_FLAG_C))
				iph2->flags |= ISAKMP_FLAG_C;

			/* call main process of quick mode */
			if (quick_main(iph2, msg) < 0) {
				plog(LLV_ERROR, LOCATION, iph1->remote,
				     "phase2 negotiation failed.\n");
				unbindph12(iph2);
				remph2(iph2);
				delph2(iph2);
				return -1;
			}
		}
		break;

	case ISAKMP_ETYPE_NEWGRP:
		if (iph1 == NULL) {
			plog(LLV_ERROR, LOCATION, remote,
			     "Unknown new group mode exchange, "
			     "there is no ISAKMP-SA.\n");
			++isakmpstat.unknown_cookie;
			return -1;
		}
		isakmp_newgroup_r(iph1, msg);
		break;

	case ISAKMP_ETYPE_NONE:
	default:
		plog(LLV_ERROR, LOCATION, NULL,
		     "Invalid exchange type %d from %s.\n",
		     isakmp->etype, saddr2str(remote));
		++isakmpstat.unsupported_exchange_type;
		return -1;
	}

	return 0;
}

/*
 * main function of phase 1.
 */
static int
ph1_main(iph1, msg)
	struct ph1handle *iph1;
	vchar_t *msg;
{
	int error;
#ifdef ENABLE_STATS
	struct timeval start, end;
#endif

	/* ignore a packet */
	if (iph1->status == PHASE1ST_ESTABLISHED) {
		++isakmpstat.ignore;
		return 0;
	}
#ifdef ENABLE_STATS
	gettimeofday(&start, NULL);
#endif
	/* receive */
	if (ph1exchange[etypesw1(iph1->etype)]
	    [iph1->side]
	    [iph1->status] == NULL) {
		plog(LLV_ERROR, LOCATION, iph1->remote,
		     "why isn't the function defined.\n");
		++isakmpstat.ignore;
		return -1;
	}
	error = (ph1exchange[etypesw1(iph1->etype)]
		 [iph1->side]
		 [iph1->status]) (iph1, msg);
	if (error != 0) {
#if 0
		/* XXX
		 * When an invalid packet is received on phase1, it should
		 * be selected to process this packet.  That is to respond
		 * with a notify and delete phase 1 handler, OR not to respond
		 * and keep phase 1 handler.
		 */
		plog(LLV_ERROR, LOCATION, iph1->remote,
		     "failed to pre-process packet.\n");
		return -1;
#else
		/* ignore the error and keep phase 1 handler */
		return 0;
#endif
	}

	/* free resend buffer */
	if (iph1->sendbuf == NULL) {
		plog(LLV_ERROR, LOCATION, NULL, "no buffer found as sendbuf\n");
		return -1;
	}
	VPTRINIT(iph1->sendbuf);

	/* turn off schedule */
	if (iph1->scr)
		SCHED_KILL(iph1->scr);

	/* send */
	plog(LLV_DEBUG, LOCATION, NULL, "===\n");
	if ((ph1exchange[etypesw1(iph1->etype)]
	     [iph1->side]
	     [iph1->status]) (iph1, msg) != 0) {
		plog(LLV_ERROR, LOCATION, iph1->remote,
		     "failed to process packet.\n");
		return -1;
	}
#ifdef ENABLE_STATS
	gettimeofday(&end, NULL);
	syslog(LOG_NOTICE, "%s(%s): %8.6f",
	       "phase1", s_isakmp_state(iph1->etype, iph1->side, iph1->status),
	       timedelta(&start, &end));
#endif
	if (iph1->status == PHASE1ST_ESTABLISHED) {
		++isakmpstat.ph1established;
#ifdef ENABLE_STATS
		gettimeofday(&iph1->end, NULL);
		syslog(LOG_NOTICE, "%s(%s): %8.6f",
		       "phase1", s_isakmp_etype(iph1->etype),
		       timedelta(&iph1->start, &iph1->end));
#endif

		/* save created date. */
		(void)time(&iph1->created);

		/* add to the schedule to expire, and seve back pointer. */
		iph1->sce = sched_new(iph1->approval->lifetime,
				      isakmp_ph1expire_stub, iph1);

		/* INITIAL-CONTACT processing */
		/* don't anything if local test mode. */
		if (!f_local
		    && iph1->rmconf->ikev1
		    && iph1->rmconf->ikev1->ini_contact
		    && !getcontacted(iph1->remote)) {
			++isakmpstat.initial_contact;
			/* insert a node into contacted list. */
			if (inscontacted(iph1->remote) == -1) {
				plog(LLV_ERROR, LOCATION, iph1->remote,
				     "failed to add contacted list.\n");
				/* ignore */
			} else {
				/* send INITIAL-CONTACT */
				isakmp_info_send_n1(iph1,
						    ISAKMP_NTYPE_INITIAL_CONTACT,
						    NULL);
			}
		}

		log_ph1established(iph1);
		plog(LLV_DEBUG, LOCATION, NULL, "===\n");
	}

	return 0;
}

/*
 * main function of quick mode.
 */
static int
quick_main(iph2, msg)
	struct ph2handle *iph2;
	vchar_t *msg;
{
	struct isakmp *isakmp = (struct isakmp *)msg->v;
	int error;
#ifdef ENABLE_STATS
	struct timeval start, end;
#endif

	/* ignore a packet */
	if (iph2->status == PHASE2ST_ESTABLISHED
	    || iph2->status == PHASE2ST_GETSPISENT)
		return 0;

#ifdef ENABLE_STATS
	gettimeofday(&start, NULL);
#endif

	/* receive */
	if (ph2exchange[etypesw2(isakmp->etype)]
	    [iph2->side]
	    [iph2->status] == NULL) {
		plog(LLV_ERROR, LOCATION, iph2->ph1->remote,
		     "why isn't the function defined.\n");
		return -1;
	}
	error = (ph2exchange[etypesw2(isakmp->etype)]
		 [iph2->side]
		 [iph2->status]) (iph2, msg);
	if (error != 0) {
		plog(LLV_ERROR, LOCATION, iph2->ph1->remote,
		     "failed to pre-process packet.\n");
		if (error == ISAKMP_INTERNAL_ERROR)
			return 0;
		isakmp_info_send_n1(iph2->ph1, error, NULL);
		return -1;
	}

	/* when using commit bit, status will be reached here. */
	if (iph2->status == PHASE2ST_ADDSA)
		return 0;

	/* free resend buffer */
	if (iph2->sendbuf == NULL) {
		plog(LLV_ERROR, LOCATION, NULL, "no buffer found as sendbuf\n");
		return -1;
	}
	VPTRINIT(iph2->sendbuf);

	/* turn off schedule */
	if (iph2->scr)
		SCHED_KILL(iph2->scr);

	/* send */
	plog(LLV_DEBUG, LOCATION, NULL, "===\n");
	if ((ph2exchange[etypesw2(isakmp->etype)]
	     [iph2->side]
	     [iph2->status]) (iph2, msg) != 0) {
		plog(LLV_ERROR, LOCATION, iph2->ph1->remote,
		     "failed to process packet.\n");
		return -1;
	}
#ifdef ENABLE_STATS
	gettimeofday(&end, NULL);
	syslog(LOG_NOTICE, "%s(%s): %8.6f",
	       "phase2",
	       s_isakmp_state(ISAKMP_ETYPE_QUICK, iph2->side, iph2->status),
	       timedelta(&start, &end));
#endif

	return 0;
}

/* new negotiation of phase 1 for initiator */
int
isakmp_ph1begin_i(rmconf, remote, local)
	struct remoteconf *rmconf;
	struct sockaddr *remote, *local;
{
	struct ph1handle *iph1;
#ifdef ENABLE_STATS
	struct timeval start, end;
#endif

	/* get new entry to isakmp status table. */
	iph1 = newph1();
	if (iph1 == NULL)
		return -1;

	iph1->status = PHASE1ST_START;
	iph1->rmconf = rmconf;
	iph1->side = INITIATOR;
	iph1->version = ISAKMP_VERSION_NUMBER;
	iph1->msgid = 0;
	iph1->flags = 0;
	iph1->ph2cnt = 0;
#ifdef HAVE_GSSAPI
	iph1->gssapi_state = NULL;
#endif
	iph1->approval = NULL;

	/* XXX copy remote address */
	if (copy_ph1addresses(iph1, rmconf, remote, local) < 0)
		return -1;

	(void)insph1(iph1);

	/* start phase 1 exchange */
	iph1->etype = conf_exmode_to_isakmp(rmconf->ikev1->ex_mode);

	plog(LLV_DEBUG, LOCATION, NULL, "===\n");
	{
		char *a;

		a = strdup(saddr2str(iph1->local));
		plog(LLV_INFO, LOCATION, NULL,
		     "initiate new phase 1 negotiation: %s<=>%s\n",
		     a, saddr2str(iph1->remote));
		racoon_free(a);
	}
	plog(LLV_INFO, LOCATION, NULL,
	     "begin %s mode.\n", s_isakmp_etype(iph1->etype));

#ifdef ENABLE_STATS
	gettimeofday(&iph1->start, NULL);
	gettimeofday(&start, NULL);
#endif
	/* start exchange */
	if ((ph1exchange[etypesw1(iph1->etype)]
	     [iph1->side]
	     [iph1->status]) (iph1, NULL) != 0) {
		/* failed to start phase 1 negotiation */
		remph1(iph1);
		delph1(iph1);

		return -1;
	}
#ifdef ENABLE_STATS
	gettimeofday(&end, NULL);
	syslog(LOG_NOTICE, "%s(%s): %8.6f",
	       "phase1",
	       s_isakmp_state(iph1->etype, iph1->side, iph1->status),
	       timedelta(&start, &end));
#endif

	return 0;
}

/* new negotiation of phase 1 for responder */
static int
isakmp_ph1begin_r(msg, remote, local, etype)
	vchar_t *msg;
	struct sockaddr *remote, *local;
	u_int8_t etype;
{
	struct isakmp *isakmp = (struct isakmp *)msg->v;
	struct remoteconf *rmconf;
	struct ph1handle *iph1;
	struct etypes *etypeok;
#ifdef ENABLE_STATS
	struct timeval start, end;
#endif

	/* look for my configuration */
	rmconf = getrmconf(remote);
	if (rmconf == NULL) {
		plog(LLV_ERROR, LOCATION, remote,
		     "couldn't find " "configuration.\n");
		return -1;
	}

	/* check to be acceptable exchange type */
	if (etype != conf_exmode_to_isakmp(rmconf->ikev1->ex_mode)) {
		plog(LLV_ERROR, LOCATION, remote,
		     "not acceptable %s mode\n", s_isakmp_etype(etype));
		return -1;
	}

	/* get new entry to isakmp status table. */
	iph1 = newph1();
	if (iph1 == NULL)
		return -1;

	memcpy(&iph1->index.i_ck, &isakmp->i_ck, sizeof(iph1->index.i_ck));
	iph1->status = PHASE1ST_START;
	iph1->rmconf = rmconf;
	iph1->flags = 0;
	iph1->side = RESPONDER;
	iph1->etype = etypeok->type;
	iph1->version = isakmp->v;
	iph1->msgid = 0;
#ifdef HAVE_GSSAPI
	iph1->gssapi_state = NULL;
#endif
	iph1->approval = NULL;

	/* copy remote address */
	if (copy_ph1addresses(iph1, rmconf, remote, local) < 0)
		return -1;

	(void)insph1(iph1);

	plog(LLV_DEBUG, LOCATION, NULL, "===\n");
	{
		char *a;

		a = strdup(saddr2str(iph1->local));
		plog(LLV_INFO, LOCATION, NULL,
		     "respond new phase 1 negotiation: %s<=>%s\n",
		     a, saddr2str(iph1->remote));
		racoon_free(a);
	}
	plog(LLV_INFO, LOCATION, NULL,
	     "begin %s mode.\n", s_isakmp_etype(etype));

#ifdef ENABLE_STATS
	gettimeofday(&iph1->start, NULL);
	gettimeofday(&start, NULL);
#endif
	/* start exchange */
	if ((ph1exchange[etypesw1(iph1->etype)]
	     [iph1->side]
	     [iph1->status]) (iph1, msg) < 0
	    || (ph1exchange[etypesw1(iph1->etype)]
		[iph1->side]
		[iph1->status]) (iph1, msg) < 0) {
		plog(LLV_ERROR, LOCATION, remote,
		     "failed to process packet.\n");
		remph1(iph1);
		delph1(iph1);
		return -1;
	}
#ifdef ENABLE_STATS
	gettimeofday(&end, NULL);
	syslog(LOG_NOTICE, "%s(%s): %8.6f",
	       "phase1",
	       s_isakmp_state(iph1->etype, iph1->side, iph1->status),
	       timedelta(&start, &end));
#endif

	return 0;
}

/* new negotiation of phase 2 for initiator */
static int
isakmp_ph2begin_i(iph1, iph2)
	struct ph1handle *iph1;
	struct ph2handle *iph2;
{
	/* found ISAKMP-SA. */
	plog(LLV_DEBUG, LOCATION, NULL, "===\n");
	plog(LLV_DEBUG, LOCATION, NULL, "begin QUICK mode.\n");
	{
		char *a;
		a = strdup(saddr2str(iph2->src));
		plog(LLV_INFO, LOCATION, NULL,
		     "initiate new phase 2 negotiation: %s<=>%s\n",
		     a, saddr2str(iph2->dst));
		racoon_free(a);
	}

#ifdef ENABLE_STATS
	gettimeofday(&iph2->start, NULL);
#endif
	/* found isakmp-sa */
	bindph12(iph1, iph2);
	iph2->status = PHASE2ST_STATUS2;

	if ((ph2exchange[etypesw2(ISAKMP_ETYPE_QUICK)]
	     [iph2->side]
	     [iph2->status]) (iph2, NULL) < 0) {
		unbindph12(iph2);
		/* release ipsecsa handler due to internal error. */
		remph2(iph2);
		delph2(iph2);
		return -1;
	}
	return 0;
}

/* new negotiation of phase 2 for responder */
static int
isakmp_ph2begin_r(iph1, msg)
	struct ph1handle *iph1;
	vchar_t *msg;
{
	struct isakmp *isakmp = (struct isakmp *)msg->v;
	struct ph2handle *iph2 = 0;
	int error;
#ifdef ENABLE_STATS
	struct timeval start, end;
#endif

	iph2 = newph2();
	if (iph2 == NULL) {
		plog(LLV_ERROR, LOCATION, NULL,
		     "failed to allocate phase2 entry.\n");
		return -1;
	}

	iph2->ph1 = iph1;
	iph2->side = RESPONDER;
	iph2->status = PHASE2ST_START;
	iph2->flags = isakmp->flags;
	iph2->msgid = isakmp->msgid;
	iph2->seq = pk_getseq();
	iph2->ivm = oakley_newiv2(iph1, iph2->msgid);
	if (iph2->ivm == NULL) {
		delph2(iph2);
		return -1;
	}
	iph2->dst = dupsaddr(iph1->remote);	/* XXX should be considered */
	if (iph2->dst == NULL) {
		delph2(iph2);
		return -1;
	}
	switch (iph2->dst->sa_family) {
	case AF_INET:
		((struct sockaddr_in *)iph2->dst)->sin_port = 0;
		break;
#ifdef INET6
	case AF_INET6:
		((struct sockaddr_in6 *)iph2->dst)->sin6_port = 0;
		break;
#endif
	default:
		plog(LLV_ERROR, LOCATION, NULL,
		     "invalid family: %d\n", iph2->dst->sa_family);
		delph2(iph2);
		return -1;
	}

	iph2->src = dupsaddr(iph1->local);	/* XXX should be considered */
	if (iph2->src == NULL) {
		delph2(iph2);
		return -1;
	}
	switch (iph2->src->sa_family) {
	case AF_INET:
		((struct sockaddr_in *)iph2->src)->sin_port = 0;
		break;
#ifdef INET6
	case AF_INET6:
		((struct sockaddr_in6 *)iph2->src)->sin6_port = 0;
		break;
#endif
	default:
		plog(LLV_ERROR, LOCATION, NULL,
		     "invalid family: %d\n", iph2->src->sa_family);
		delph2(iph2);
		return -1;
	}

	/* add new entry to isakmp status table */
	insph2(iph2);
	bindph12(iph1, iph2);

	plog(LLV_DEBUG, LOCATION, NULL, "===\n");
	{
		char *a;

		a = strdup(saddr2str(iph2->src));
		plog(LLV_INFO, LOCATION, NULL,
		     "respond new phase 2 negotiation: %s<=>%s\n",
		     a, saddr2str(iph2->dst));
		racoon_free(a);
	}

#ifdef ENABLE_STATS
	gettimeofday(&start, NULL);
#endif

	error = (ph2exchange[etypesw2(ISAKMP_ETYPE_QUICK)]
		 [iph2->side]
		 [iph2->status]) (iph2, msg);
	if (error != 0) {
		plog(LLV_ERROR, LOCATION, iph1->remote,
		     "failed to pre-process packet.\n");
		if (error != ISAKMP_INTERNAL_ERROR)
			isakmp_info_send_n1(iph2->ph1, error, NULL);
		/*
		 * release handler because it's wrong that ph2handle is kept
		 * after failed to check message for responder's.
		 */
		unbindph12(iph2);
		remph2(iph2);
		delph2(iph2);
		return -1;
	}

	/* send */
	plog(LLV_DEBUG, LOCATION, NULL, "===\n");
	if ((ph2exchange[etypesw2(isakmp->etype)]
	     [iph2->side]
	     [iph2->status]) (iph2, msg) < 0) {
		plog(LLV_ERROR, LOCATION, iph2->ph1->remote,
		     "failed to process packet.\n");
		/* don't release handler */
		return -1;
	}
#ifdef ENABLE_STATS
	gettimeofday(&end, NULL);
	syslog(LOG_NOTICE, "%s(%s): %8.6f",
	       "phase2",
	       s_isakmp_state(ISAKMP_ETYPE_QUICK, iph2->side, iph2->status),
	       timedelta(&start, &end));
#endif

	return 0;
}

/* called from scheduler */
static void
isakmp_ph1resend_stub(p)
	void *p;
{
	(void)isakmp_ph1resend((struct ph1handle *)p);
}

int
isakmp_ph1resend(iph1)
	struct ph1handle *iph1;
{
	if (iph1->retry_counter < 0) {
		plog(LLV_ERROR, LOCATION, NULL,
		     "phase1 negotiation failed due to time up. %s\n",
		     isakmp_pindex(&iph1->index, iph1->msgid));

		remph1(iph1);
		delph1(iph1);
		return -1;
	}

	if (isakmp_send(iph1, iph1->sendbuf) < 0)
		return -1;

	plog(LLV_DEBUG, LOCATION, NULL,
	     "resend phase1 packet %s\n",
	     isakmp_pindex(&iph1->index, iph1->msgid));

	iph1->retry_counter--;

	iph1->scr = sched_new(iph1->rmconf->retry_interval,
			      isakmp_ph1resend_stub, iph1);

	return 0;
}

/* called from scheduler */
static void
isakmp_ph2resend_stub(p)
	void *p;
{

	(void)isakmp_ph2resend((struct ph2handle *)p);
}

int
isakmp_ph2resend(iph2)
	struct ph2handle *iph2;
{
	if (iph2->retry_counter < 0) {
		plog(LLV_ERROR, LOCATION, NULL,
		     "phase2 negotiation failed due to time up. %s\n",
		     isakmp_pindex(&iph2->ph1->index, iph2->msgid));
		unbindph12(iph2);
		remph2(iph2);
		delph2(iph2);
		return -1;
	}

	if (isakmp_send(iph2->ph1, iph2->sendbuf) < 0)
		return -1;

	plog(LLV_DEBUG, LOCATION, NULL,
	     "resend phase2 packet %s\n",
	     isakmp_pindex(&iph2->ph1->index, iph2->msgid));

	iph2->retry_counter--;

	iph2->scr = sched_new(iph2->ph1->rmconf->retry_interval,
			      isakmp_ph2resend_stub, iph2);

	return 0;
}

/* called from scheduler */
static void
isakmp_ph1expire_stub(p)
	void *p;
{

	isakmp_ph1expire((struct ph1handle *)p);
}

void
isakmp_ph1expire(iph1)
	struct ph1handle *iph1;
{
	char *src, *dst;

	src = strdup(saddr2str(iph1->local));
	dst = strdup(saddr2str(iph1->remote));
	plog(LLV_INFO, LOCATION, NULL,
	     "ISAKMP-SA expired %s-%s spi:%s\n",
	     src, dst, isakmp_pindex(&iph1->index, 0));
	racoon_free(src);
	racoon_free(dst);

	SCHED_KILL(iph1->sce);

	iph1->status = PHASE1ST_EXPIRED;

	/*
	 * the phase1 deletion is postponed until there is no phase2.
	 */
	if (LIST_FIRST(&iph1->ph2tree) != NULL) {
		iph1->sce = sched_new(1, isakmp_ph1expire_stub, iph1);
		return;
	}

	iph1->sce = sched_new(1, isakmp_ph1delete_stub, iph1);
}

/* called from scheduler */
static void
isakmp_ph1delete_stub(p)
	void *p;
{

	isakmp_ph1delete((struct ph1handle *)p);
}

void
isakmp_ph1delete(iph1)
	struct ph1handle *iph1;
{
	char *src, *dst;

	SCHED_KILL(iph1->sce);

	if (LIST_FIRST(&iph1->ph2tree) != NULL) {
		iph1->sce = sched_new(1, isakmp_ph1delete_stub, iph1);
		return;
	}

	/* don't re-negosiation when the phase 1 SA expires. */

	src = strdup(saddr2str(iph1->local));
	dst = strdup(saddr2str(iph1->remote));
	plog(LLV_INFO, LOCATION, NULL,
	     "ISAKMP-SA deleted %s-%s spi:%s\n",
	     src, dst, isakmp_pindex(&iph1->index, 0));
	racoon_free(src);
	racoon_free(dst);

	remph1(iph1);
	delph1(iph1);

	return;
}

/* called from scheduler.
 * this function will call only isakmp_ph2delete().
 * phase 2 handler remain forever if kernel doesn't cry a expire of phase 2 SA
 * by something cause.  That's why this function is called after phase 2 SA
 * expires in the userland.
 */
static void
isakmp_ph2expire_stub(p)
	void *p;
{
	isakmp_ph2expire((struct ph2handle *)p);
}

void
isakmp_ph2expire(iph2)
	struct ph2handle *iph2;
{
	char *src, *dst;

	SCHED_KILL(iph2->sce);

	src = strdup(saddrwop2str(iph2->src));
	dst = strdup(saddrwop2str(iph2->dst));
	plog(LLV_INFO, LOCATION, NULL, "phase2 sa expired %s-%s\n", src, dst);
	racoon_free(src);
	racoon_free(dst);

	iph2->status = PHASE2ST_EXPIRED;

	iph2->sce = sched_new(1, isakmp_ph2delete_stub, iph2);

	return;
}

/* called from scheduler */
void
isakmp_ph2delete_stub(p)
	void *p;
{

	isakmp_ph2delete((struct ph2handle *)p);
}

void
isakmp_ph2delete(iph2)
	struct ph2handle *iph2;
{
	char *src, *dst;

	SCHED_KILL(iph2->sce);

	src = strdup(saddrwop2str(iph2->src));
	dst = strdup(saddrwop2str(iph2->dst));
	plog(LLV_INFO, LOCATION, NULL, "phase2 sa deleted %s-%s\n", src, dst);
	racoon_free(src);
	racoon_free(dst);

	unbindph12(iph2);
	remph2(iph2);
	delph2(iph2);

	return;
}

/*
 * conversion routine for use with dispatch tables
 */
static int
etypesw1(etype)
	int etype;
{
	switch (etype) {
	case ISAKMP_ETYPE_IDENT:
		return 1;
	case ISAKMP_ETYPE_AGG:
		return 2;
	case ISAKMP_ETYPE_BASE:
		return 3;
	default:
		return 0;
	}
 /*NOTREACHED*/}

static int
etypesw2(etype)
	int etype;
{
	switch (etype) {
	case ISAKMP_ETYPE_QUICK:
		return 1;
	default:
		return 0;
	}
 /*NOTREACHED*/}

int
copy_ph1addresses(iph1, rmconf, remote, local)
	struct ph1handle *iph1;
	struct remoteconf *rmconf;
	struct sockaddr *remote, *local;
{
	u_short *port = NULL;

	/* address portion must be grabbed from real remote address "remote" */
	iph1->remote = dupsaddr(remote);
	if (iph1->remote == NULL) {
		delph1(iph1);
		return -1;
	}

	/*
	 * if remote has no port # (in case of initiator - from ACQUIRE msg)
	 * - if remote.conf specifies port #, use that
	 * - if remote.conf does not, use 500
	 * if remote has port # (in case of responder - from recvfrom(2))
	 * respect content of "remote".
	 */
	switch (iph1->remote->sa_family) {
	case AF_INET:
		port = &((struct sockaddr_in *)iph1->remote)->sin_port;
		if (*port)
			break;
		*port = ((struct sockaddr_in *)rmconf->remote)->sin_port;
		if (*port)
			break;
		*port = htons(PORT_ISAKMP);
		break;
#ifdef INET6
	case AF_INET6:
		port = &((struct sockaddr_in6 *)iph1->remote)->sin6_port;
		if (*port)
			break;
		*port = ((struct sockaddr_in6 *)rmconf->remote)->sin6_port;
		if (*port)
			break;
		*port = htons(PORT_ISAKMP);
		break;
#endif
	default:
		plog(LLV_ERROR, LOCATION, NULL,
		     "invalid family: %d\n", iph1->remote->sa_family);
		return -1;
	}

	if (local == NULL)
		iph1->local = getlocaladdr(iph1->remote);
	else
		iph1->local = dupsaddr(local);
	if (iph1->local == NULL) {
		delph1(iph1);
		return -1;
	}
	switch (iph1->local->sa_family) {
	case AF_INET:
		((struct sockaddr_in *)iph1->local)->sin_port
			= getmyaddrsport(iph1->local);
		break;
#ifdef INET6
	case AF_INET6:
		((struct sockaddr_in6 *)iph1->local)->sin6_port
			= getmyaddrsport(iph1->local);
		break;
#endif
	default:
		plog(LLV_ERROR, LOCATION, NULL,
		     "invalid family: %d\n", iph1->remote->sa_family);
		delph1(iph1);
		return -1;
	}

	return 0;
}

static int
nostate1(iph1, msg)
	struct ph1handle *iph1;
	vchar_t *msg;
{
	plog(LLV_ERROR, LOCATION, iph1->remote, "wrong state %u.\n",
	     iph1->status);
	return -1;
}

static int
nostate2(iph2, msg)
	struct ph2handle *iph2;
	vchar_t *msg;
{
	plog(LLV_ERROR, LOCATION, iph2->ph1->remote, "wrong state %u.\n",
	     iph2->status);
	return -1;
}

void
log_ph1established(iph1)
	const struct ph1handle *iph1;
{
	char *src, *dst;

	src = strdup(saddr2str(iph1->local));
	dst = strdup(saddr2str(iph1->remote));
	plog(LLV_INFO, LOCATION, NULL,
	     "ISAKMP-SA established %s-%s spi:%s\n",
	     src, dst, isakmp_pindex(&iph1->index, 0));
	racoon_free(src);
	racoon_free(dst);

	return;
}

/*
 * calculate cookie and set.
 */
int
isakmp_newcookie(place, remote, local)
	caddr_t place;
	struct sockaddr *remote;
	struct sockaddr *local;
{
	vchar_t *buf = NULL, *buf2 = NULL;
	char *p;
	int blen;
	int alen;
	caddr_t sa1, sa2;
	time_t t;
	int error = -1;
	u_short port;

	if (remote->sa_family != local->sa_family) {
		plog(LLV_ERROR, LOCATION, NULL,
		     "address family mismatch, remote:%d local:%d\n",
		     remote->sa_family, local->sa_family);
		goto end;
	}
	switch (remote->sa_family) {
	case AF_INET:
		alen = sizeof(struct in_addr);
		sa1 = (caddr_t)&((struct sockaddr_in *)remote)->sin_addr;
		sa2 = (caddr_t)&((struct sockaddr_in *)local)->sin_addr;
		break;
#ifdef INET6
	case AF_INET6:
		alen = sizeof(struct in_addr);
		sa1 = (caddr_t)&((struct sockaddr_in6 *)remote)->sin6_addr;
		sa2 = (caddr_t)&((struct sockaddr_in6 *)local)->sin6_addr;
		break;
#endif
	default:
		plog(LLV_ERROR, LOCATION, NULL,
		     "invalid family: %d\n", remote->sa_family);
		goto end;
	}
	blen = (alen + sizeof(u_short)) * 2
		+ sizeof(time_t) + lcconf->secret_size;
	buf = vmalloc(blen);
	if (buf == NULL) {
		plog(LLV_ERROR, LOCATION, NULL, "failed to get a cookie.\n");
		goto end;
	}
	p = buf->v;

	/* copy my address */
	memcpy(p, sa1, alen);
	p += alen;
	port = ((struct sockaddr_in *)remote)->sin_port;
	memcpy(p, &port, sizeof(u_short));
	p += sizeof(u_short);

	/* copy target address */
	memcpy(p, sa2, alen);
	p += alen;
	port = ((struct sockaddr_in *)local)->sin_port;
	memcpy(p, &port, sizeof(u_short));
	p += sizeof(u_short);

	/* copy time */
	t = time(0);
	memcpy(p, (caddr_t)&t, sizeof(t));
	p += sizeof(t);

	/* copy random value */
	buf2 = eay_set_random(lcconf->secret_size);
	if (buf2 == NULL)
		goto end;
	memcpy(p, buf2->v, lcconf->secret_size);
	p += lcconf->secret_size;
	vfree(buf2);

	buf2 = eay_sha1_one(buf);
	memcpy(place, buf2->v, sizeof(isakmp_cookie_t));

	sa1 = val2str(place, sizeof(isakmp_cookie_t));
	plog(LLV_DEBUG, LOCATION, NULL, "new cookie:\n%s\n", sa1);
	racoon_free(sa1);

	error = 0;
      end:
	if (buf != NULL)
		vfree(buf);
	if (buf2 != NULL)
		vfree(buf2);
	return error;
}

/*
 * save partner's(payload) data into phhandle.
 */
int
isakmp_p2ph(buf, gen)
	vchar_t **buf;
	struct isakmp_gen *gen;
{
	/* XXX to be checked in each functions for logging. */
	if (*buf) {
		plog(LLV_WARNING, LOCATION, NULL,
		     "ignore this payload, same payload type exist.\n");
		return -1;
	}

	*buf = vmalloc(ntohs(gen->len) - sizeof(*gen));
	if (*buf == NULL) {
		plog(LLV_ERROR, LOCATION, NULL, "failed to get buffer.\n");
		return -1;
	}
	memcpy((*buf)->v, gen + 1, (*buf)->l);

	return 0;
}

static int
check_spi_size(proto_id, size)
	int proto_id, size;
{
	switch (proto_id) {
	case IPSECDOI_PROTO_ISAKMP:
		if (size != 0) {
			/* WARNING */
			plog(LLV_DEBUG, LOCATION, NULL,
			     "SPI size isn't zero, but IKE proposal.\n");
		}
		return 0;

	case IPSECDOI_PROTO_IPSEC_AH:
	case IPSECDOI_PROTO_IPSEC_ESP:
		if (size != 4) {
			plog(LLV_ERROR, LOCATION, NULL,
			     "invalid SPI size=%d for IPSEC proposal.\n", size);
			return -1;
		}
		return 0;

	case IPSECDOI_PROTO_IPCOMP:
		if (size != 2 && size != 4) {
			plog(LLV_ERROR, LOCATION, NULL,
			     "invalid SPI size=%d for IPCOMP proposal.\n",
			     size);
			return -1;
		}
		return 0;

	default:
		/* ??? */
		return -1;
	}
	/* NOT REACHED */
}
